diff --git a/FHEM/57_Calendar.pm b/FHEM/57_Calendar.pm
index 4d05623..cc0eb58 100644
--- a/FHEM/57_Calendar.pm
+++ b/FHEM/57_Calendar.pm
@@ -332,6 +332,12 @@ sub fromVEvent {
   $self->{lastModified}= tm($vevent->value("LAST-MODIFIED"));
   $self->{summary}= $vevent->value("SUMMARY");
   $self->{location}= $vevent->value("LOCATION");
+
+  #Dates to exclude in reoccuring rule
+  my @exdate = split(",", $vevent->value("EXDATE"));
+  @exdate = map { tm($_) } @exdate;
+  $self->{exdate} = \@exdate;
+
   #$self->{summary}=~ s/;/,/g;
 
   #
@@ -343,16 +349,22 @@ sub fromVEvent {
   if($rrule) {
     my @rrparts= split(";", $rrule);
     my %r= map { split("=", $_); } @rrparts;
-    #foreach my $k (keys %r) {
-    #  main::debug "Rule part $k is $r{$k}";
-    #}
-    my $freq= $r{"FREQ"};
-    #
-    # weekly
-    #
-    if($freq eq "WEEKLY") {
-      # my @weekdays= split(",",$r{"BYDAY"});# BYDAY is not always set
+
+    foreach my $k (keys %r) {
+      if( $k ne "FREQ" and $k ne "INTERVAL" and $k ne "UNTIL" and $k ne "COUNT") {
+        main::Log 2, "Calendar: RRULE $rrule is not supported";
+      }
     }
+
+    $self->{freq} =  $r{"FREQ"};
+    #According to RFC, interval defaults to 1
+    $self->{interval} = exists($r{"INTERVAL"}) ? $r{"INTERVAL"} : 1;
+    $self->{until} = tm($r{"UNTIL"}) if(exists($r{"UNTIL"}));
+    $self->{count} = $r{"COUNT"} if(exists($r{"COUNT"}));
+
+    # advanceToNextOccurance until we are in the future
+    my $t = time();
+    while($self->{end} < $t and $self->advanceToNextOccurance()) { ; }
   }
   
 
@@ -427,6 +439,48 @@ sub endTime {
   return ts($self->{end});
 }
 
+sub advanceToNextOccurance {
+  my ($self) = @_;
+  # See RFC 2445 page 39 and following
+
+  return if(!exists($self->{freq})); #This event is not reoccuring
+  return if(exists($self->{count}) and $self->{count} == 0); #We are already at the last occurance
+
+  #There are no leap seconds in epoch time
+  #Valid values for freq: SECONDLY, MINUTELY, HOURLY, DAILY, WEEKLY, MONTHLY, YEARLY
+  my $nextstart = $self->{start};
+  do
+  {
+    if($self->{freq} eq "SECONDLY") {
+      $nextstart += $self->{interval};
+    } elsif($self->{freq} eq "MINUTELY") {
+      $nextstart += 60*$self->{interval};
+    } elsif($self->{freq} eq "HOURLY") {
+      $nextstart += 60*60*$self->{interval};
+    } elsif($self->{freq} eq "DAILY") {
+      $nextstart += 60*60*24*$self->{interval};
+    } elsif($self->{freq} eq "WEEKLY") {
+      $nextstart += 7*60*60*24*$self->{interval};
+    } else {
+      #TODO: with MONTHLY and YEARLY, one has to account for leap days
+      main::Log 1, "Calendar: event frequency '" . $self->{freq} . "' not implemented";
+      return;
+    }
+
+  # Loop if nextstart is in the "dates to exclude"
+  } while(exists($self->{exdate}) and ($nextstart ~~ $self->{exdate}));
+
+  #the UNTIL clause is inclusive, so $newt == $self->{until} is okey
+  return if(exists($self->{until}) and $nextstart > $self->{until});
+  $self->{count} -= 1 if(exists($self->{count}));
+
+  my $duration = $self->{end} - $self->{start};
+  $self->{start} = $nextstart;
+  $self->{end} = $self->{start} + $duration;
+  main::Log 5, "Next time of $self->{summary} is: start " . $self->{"start"} . ", end " . $self->{"end"};
+  return 1;
+}
+
 
 # returns 1 if time is before alarm time and before start time, else 0
 sub isUpcoming {
@@ -640,10 +694,12 @@ sub Calendar_CheckTimes($) {
 
   # we now run over all events and update the readings 
   my @allevents= $eventsObj->events();
+  my @endedevents= grep { $_->isEnded($t) } @allevents;
+  foreach (@endedevents) { $_->advanceToNextOccurance(); }
+
   my @upcomingevents= grep { $_->isUpcoming($t) } @allevents;
   my @alarmedevents= grep { $_->isAlarmed($t) } @allevents;
   my @startedevents= grep { $_->isStarted($t) } @allevents;
-  my @endedevents= grep { $_->isEnded($t) } @allevents;
 
   my $event;
   #main::debug "Updating modes...";
